(******************************************************************************)
(*   __               ___     CAVE: Concurrent Algorithm VErifier             *)
(*  /     /\  \    / |        Copyright (c) Viktor Vafeiadis                  *)
(* |     /--\  \  /  |---                                                     *)
(*  \__ /    \  \/   |___     See LICENSE.txt for license.                    *)
(*                                                                            *)
(******************************************************************************)
(** Linearizability *)

(** This module implements two linearizability verifiers.

    References:
     - Vafeiadis, V.: Automatically proving linearizability.
       In CAV 2010. Springer, Heidelberg (2010)
     - Henzinger, T. A., Sezgin, A., Vafeiadis, V.: Aspect-oriented linearizability proofs.
       In CONCUR 2013. LNCS, vol. 8052. Springer, Heidelberg (August 2013)
 *)

(** Check validity of program specifications. *)
val check_props : 
  Assertions.prover -> Genarith.abstraction ->
  string list *
  Misc.StringSet.t *
  Exp.IdSet.t *
  (string, Tycheck.protocol_item) Hashtbl.t *
  (string, Tycheck.escrow_item) Hashtbl.t *
  (string, Commands.fun_info) Hashtbl.t *
  (Misc.component, Assertions.act list * Assertions.act list) Hashtbl.t *
  (Misc.component * Commands.res_init) list *
  (string * Commands.can_entailment) list -> bool

(* -------------------------------------------------------------------------- *)
(** {2 Configuration parameters & statistics} *)
(* -------------------------------------------------------------------------- *)

val verbose : int ref
val infer : int ref

val args : (Arg.key * Arg.spec * Arg.doc) list
