(******************************************************************************)
(*   __               ___     CAVE: Concurrent Algorithm VErifier             *)
(*  /     /\  \    / |        Copyright (c) Viktor Vafeiadis                  *)
(* |     /--\  \  /  |---                                                     *)
(*  \__ /    \  \/   |___     See LICENSE.txt for license.                    *)
(*                                                                            *)
(******************************************************************************)
(** Type checking & conversion into command language *)
open Assertions
open Commands

val args: (Arg.key * Arg.spec * Arg.doc) list

type state_predecessor = string * Exp.Id.t list * cprop

type protocol_state = string * Exp.Id.t list * state_predecessor list * Exp.Id.t * cprop

type protocol_item =
  { protocol_name: string
  ; protocol_states: protocol_state Misc.StringMap.t
  ; protocol_successors: Misc.StringSet.t Misc.StringMap.t }

type escrow_item =
  { escrow_name: string
  ; escrow_params: Exp.Id.t list
  ; escrow_pre: cprop
  ; escrow_post: cprop }

(** Returns 
    [(globals, protocols, fun_specifications, resources, res_initializers, fun_proofs)] *)
val convert : Parsetree.p_item list -> 
  (string list *
   Misc.StringSet.t *
   Exp.IdSet.t *
   (string, protocol_item) Hashtbl.t *
   (string, escrow_item) Hashtbl.t *
   (string, fun_info) Hashtbl.t *  
   (Misc.component, act list * act list) Hashtbl.t * 
   (Misc.component * res_init) list * 
   (string * can_entailment) list) option 
